<#
.SYNOPSIS
Removes application entries from the Windows file and folder context menus.

.DESCRIPTION
The script removes app context menu items from the new menu in Windows 11 and legacy menu in Windows 11/10.
The script takes care only of the apps registered via a manifest.
It collects extensions IDs from app manifests and blocks them via Windows Registry.

.PARAMETER Apps
Comma-separated list of app names to remove from the context menu.
If not provided, the script lists all registered apps and their CLSIDs.

.PARAMETER Scope
Accepts HKLM (all users) or HKCU (current user, default behavior).

.EXAMPLE
PS> .\context-menu.ps1
Lists all registered app names and their CLSIDs.

.EXAMPLE
PS> .\context-menu.ps1 -Apps onedrive,windowsnotepad,skype
Removes the listed apps from the current user context menu. Unique partial names are ok.

.EXAMPLE
PS> .\context-menu.ps1 -Apps onedrive,windowsnotepad,skype -Scope HKLM
Removes the listed apps from all users context menu. Unique partial names are ok.

.LINK
https://www.outsidethebox.ms/22361/
#>

param(
    [Parameter()]
    [string[]]$Apps,
    [string]$Scope = 'HKCU'
    )

# script scope - HKCU (current user, default) or HKLM (all users)  
if ( ($Scope -ne 'HKCU') -and ($Scope -ne 'HKLM') ) {
	Write-Error 'Unacceptable scope. Use HKLM or HKCU.' -ErrorAction Stop
	}

$regpath = "$($Scope):Software\Microsoft\Windows\CurrentVersion\Shell Extensions\Blocked"
New-Item -Path $(Split-Path $regpath -Parent) -Name $(Split-Path $regpath -Leaf) -ErrorAction SilentlyContinue | Out-Null

# if no app list supplied, we'll get all registered apps and CLSIDs 
if (-Not ($PSBoundParameters.ContainsKey('Apps'))) {
    Write-Host "Here's the list of context menu apps and their CLISDs:"
    $Apps = '*'
}

foreach ($app in $Apps) {
    # Get all manifests with registered context menu extensions
    $manifests = Get-ChildItem -Force "C:\Program Files\WindowsApps\*$App*\AppxManifest.xml" 
    foreach ($manifest in $manifests) {
        $xml = [xml] (Get-Content ($manifest).FullName)
        $elem = $xml.get_DocumentElement()
        #Get app name and extension CLSIDs
        $app = $elem.Identity.Name
        $clsids = $elem.Applications.Application.Extensions.Extension.comServer.SurrogateServer.class.id        
        if ($Apps -ne '*') { 
            # If apps were supplied, write CLSIDs to the registry
            foreach ($clsid in $clsids) {
            $name = '{' + $clsid + '}'
            New-ItemProperty -Path $regpath -Name $name -Type String -Value $app -Force | Out-Null
            }                     
                }
        else {
            # Output apps and their CLSIDs                
            foreach ($clsid in $clsids) {
                Write-Host "{$($clsid)} $($app)" 
            } 
        }               
    }
}
if ($Apps -ne '*') {
    # Restart explorer for the current user to apply the changes
    Get-Process explorer -IncludeUserName | where UserName -match $ENV:USERNAME | Stop-Process
    # Export REG-file to %temp%
    $regexportpath = $regpath.replace(':','\')
    reg export "$regexportpath" "$ENV:TEMP\context-menu.reg" /y
}